/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2011 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/



#include "ecg_export.h"



#if defined(__APPLE__) || defined(__MACH__) || defined(__APPLE_CC__)

#define fseeko fseek
#define ftello ftell
#define fopeno fopen

#else

#define fseeko fseeko64
#define ftello ftello64
#define fopeno fopen64

#endif




UI_ECGExport::UI_ECGExport(QWidget *parent)
{
  mainwindow = (UI_Mainwindow *)parent;

  signal_nr = -1;
}



void UI_ECGExport::Export_RR_intervals(void)
{
  int len;

  char path[1024],
       str[2048];

  FILE *outputfile;


  UI_SignalChooser Signalchooser(mainwindow, 4, &signal_nr);

  if((signal_nr < 0) || (signal_nr >= mainwindow->signalcomps))
  {
    QMessageBox messagewindow(QMessageBox::Critical, "Error", "Invalid signalcomp number");
    messagewindow.exec();
    return;
  }

  if(mainwindow->signalcomp[signal_nr]->ecg_filter == NULL)
  {
    QMessageBox messagewindow(QMessageBox::Critical, "Error", "Heart Rate detection is not activated for the selected signal.");
    messagewindow.exec();
    return;
  }

  path[0] = 0;
  if(mainwindow->recent_savedir[0]!=0)
  {
    strcpy(path, mainwindow->recent_savedir);
    strcat(path, "/");
  }
  len = strlen(path);
  get_filename_from_path(path + len, mainwindow->signalcomp[signal_nr]->edfhdr->filename, 1024 - len);
  remove_extension_from_filename(path);
  strcat(path, "_RR_interval.txt");

  strcpy(path, QFileDialog::getSaveFileName(0, "Export RR-interval to ASCII", path, "Text files (*.txt *.TXT)").toLatin1().data());

  if(!strcmp(path, ""))
  {
    return;
  }

  get_directory_from_path(mainwindow->recent_savedir, path, 1024);

  outputfile = fopeno(path, "wb");
  if(outputfile==NULL)
  {
    QMessageBox messagewindow(QMessageBox::Critical, "Error", "Error, can not open outputfile for writing.");
    messagewindow.exec();
    return;
  }

  ecg_filter_export_beats_to_ascii(mainwindow->signalcomp[signal_nr]->ecg_filter, outputfile);

//   QApplication::setOverrideCursor(Qt::WaitCursor);
//
//   ecg_filter_start_log(mainwindow->signalcomp[signal_nr]->ecg_filter, outputfile);
//
//   mainwindow->maincurve->drawCurve_stage_1();
//
//   ecg_filter_stop_log(mainwindow->signalcomp[signal_nr]->ecg_filter);
//
//   QApplication::restoreOverrideCursor();

  fclose(outputfile);

  sprintf(str, "Done. The RR-intervals are exported to:\n\n%s", path);
  QMessageBox messagewindow(QMessageBox::Information, "Ready", str);
  messagewindow.exec();
}








