/*
***************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2007, 2008, 2009, 2010, 2011 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/




#include "signal_chooser.h"



UI_SignalChooser::UI_SignalChooser(QWidget *parent, int job, int *sgnl_nr)
{
  int i;


  task = job;

  signal_nr = sgnl_nr;

  mainwindow = (UI_Mainwindow *)parent;

  signalchooser_dialog = new QDialog(parent);

  if(task == 3)
  {
    signalchooser_dialog->setMinimumSize(QSize(265, 400));
    signalchooser_dialog->setMaximumSize(QSize(265, 400));
    signalchooser_dialog->setWindowTitle("Organize signals");
  }
  else
  {
    signalchooser_dialog->setMinimumSize(QSize(200, 400));
    signalchooser_dialog->setMaximumSize(QSize(200, 400));
    signalchooser_dialog->setWindowTitle("Signals");
  }
  signalchooser_dialog->setModal(TRUE);
  signalchooser_dialog->setAttribute(Qt::WA_DeleteOnClose, TRUE);

  list = new QListWidget(signalchooser_dialog);
  list->setGeometry(5, 5, 130, 365);
  list->setSelectionBehavior(QAbstractItemView::SelectRows);
  list->setSelectionMode(QAbstractItemView::SingleSelection);


  CloseButton = new QPushButton(signalchooser_dialog);
  CloseButton->setGeometry(5, 375, 130, 20);
  CloseButton->setText("Close");

  if(task == 3)
  {
    UpButton = new QPushButton(signalchooser_dialog);
    UpButton->setGeometry(150, 180, 100, 20);
    UpButton->setText("Up");

    DownButton = new QPushButton(signalchooser_dialog);
    DownButton->setGeometry(150, 220, 100, 20);
    DownButton->setText("Down");

    InvertButton = new QPushButton(signalchooser_dialog);
    InvertButton->setGeometry(150, 260, 100, 20);
    InvertButton->setText("Invert");

    DeleteButton = new QPushButton(signalchooser_dialog);
    DeleteButton->setGeometry(150, 300, 100, 20);
    DeleteButton->setText("Remove");
  }

  for(i=0; i<mainwindow->signalcomps; i++)
  {
    new QListWidgetItem(mainwindow->signalcomp[i]->signallabel, list);
  }

  QObject::connect(CloseButton, SIGNAL(clicked()), signalchooser_dialog, SLOT(close()));
  if(task == 3)
  {
    if(list->count() > 0)
    {
      list->setCurrentRow(0);

      QObject::connect(UpButton,     SIGNAL(clicked()), this, SLOT(signalUp()));
      QObject::connect(DownButton,   SIGNAL(clicked()), this, SLOT(signalDown()));
      QObject::connect(InvertButton, SIGNAL(clicked()), this, SLOT(signalInvert()));
      QObject::connect(DeleteButton, SIGNAL(clicked()), this, SLOT(signalDelete()));
    }
  }
  else
  {
    QObject::connect(list, SIGNAL(itemClicked(QListWidgetItem *)), this, SLOT(call_sidemenu(QListWidgetItem *)));
  }

  signalchooser_dialog->exec();
}



void UI_SignalChooser::call_sidemenu(QListWidgetItem *)
{
  if(task == 3) return;

  signalchooser_dialog->hide();

  if(task == 0)
  {
    mainwindow->maincurve->exec_sidemenu(list->currentRow());
  }

  if(task == 1)
  {
    mainwindow->spectrumdock->init(list->currentRow());
  }

  if(task == 2)
  {
    AdjustFilterSettings filtersettings(mainwindow->signalcomp[list->currentRow()], mainwindow->maincurve);
  }

  if(task == 4)
  {
    if(signal_nr != NULL)
    {
      *signal_nr = list->currentRow();
    }
  }

  signalchooser_dialog->close();
}


void UI_SignalChooser::signalUp()
{
  int row,
      size;

  QListWidgetItem *item;

  struct signalcompblock * signalcomp;


  size = list->count();

  if(size < 2)
  {
    return;
  }

  row = list->currentRow();

  if(row > 0)
  {
    item = list->takeItem(row);

    list->insertItem(row - 1, item);

    list->setCurrentRow(row - 1);

    signalcomp = mainwindow->signalcomp[row];

    mainwindow->signalcomp[row] = mainwindow->signalcomp[row - 1];

    mainwindow->signalcomp[row - 1] = signalcomp;

    mainwindow->setup_viewbuf();
  }
}


void UI_SignalChooser::signalDown()
{
  int row,
      size;

  QListWidgetItem *item;

  struct signalcompblock * signalcomp;


  size = list->count();

  if(size < 2)
  {
    return;
  }

  row = list->currentRow();

  if(row < (size - 1))
  {
    item = list->takeItem(row);

    list->insertItem(row + 1, item);

    list->setCurrentRow(row + 1);

    signalcomp = mainwindow->signalcomp[row];

    mainwindow->signalcomp[row] = mainwindow->signalcomp[row + 1];

    mainwindow->signalcomp[row + 1] = signalcomp;

    mainwindow->setup_viewbuf();
  }
}


void UI_SignalChooser::signalDelete()
{
  int i, j,
      row,
      size;

  QListWidgetItem *item;


  size = list->count();

  if(size < 1)
  {
    return;
  }

  row = list->currentRow();

  item = list->takeItem(row);

  delete item;

  if(mainwindow->spectrumdock->signalcomp == mainwindow->signalcomp[row])
  {
    mainwindow->spectrumdock->clear();
    mainwindow->spectrumdock->dock->hide();
  }

  for(i=0; i<MAXSPECTRUMDIALOGS; i++)
  {
    if(mainwindow->spectrumdialog[i] != NULL)
    {
      if(mainwindow->spectrumdialog[i]->signalcomp == mainwindow->signalcomp[row])
      {
        delete mainwindow->spectrumdialog[i];

        mainwindow->spectrumdialog[i] = NULL;
      }
    }
  }

  for(i=0; i<MAXAVERAGECURVEDIALOGS; i++)
  {
    if(mainwindow->averagecurvedialog[i] != NULL)
    {
      if(mainwindow->averagecurvedialog[i]->signalcomp == mainwindow->signalcomp[row])
      {
        delete mainwindow->averagecurvedialog[i];

        mainwindow->averagecurvedialog[i] = NULL;
      }
    }
  }

  if(mainwindow->signalcomp[row]->hascursor2)
  {
    mainwindow->maincurve->crosshair_2_active = 0;
    mainwindow->maincurve->crosshair_2_moving = 0;
  }

  if(mainwindow->signalcomp[row]->hascursor1)
  {
    mainwindow->maincurve->crosshair_1_active = 0;
    mainwindow->maincurve->crosshair_2_active = 0;
    mainwindow->maincurve->crosshair_1_moving = 0;
    mainwindow->maincurve->crosshair_2_moving = 0;

    for(i=0; i<mainwindow->signalcomps; i++)
    {
      mainwindow->signalcomp[i]->hascursor2 = 0;
    }
  }

  if(mainwindow->signalcomp[row]->hasruler)
  {
    mainwindow->maincurve->ruler_active = 0;
    mainwindow->maincurve->ruler_moving = 0;
  }

  for(j=0; j<mainwindow->signalcomp[row]->filter_cnt; j++)
  {
    free(mainwindow->signalcomp[row]->filter[j]);
  }

  mainwindow->signalcomp[row]->filter_cnt = 0;

  for(j=0; j<mainwindow->signalcomp[row]->ravg_filter_cnt; j++)
  {
    free_ravg_filter(mainwindow->signalcomp[row]->ravg_filter[j]);
  }

  mainwindow->signalcomp[row]->ravg_filter_cnt = 0;

  if(mainwindow->signalcomp[row]->ecg_filter != NULL)
  {
    free_ecg_filter(mainwindow->signalcomp[row]->ecg_filter);

    mainwindow->signalcomp[row]->ecg_filter = NULL;

    mainwindow->signalcomp[row]->ecg_filter_setup = 0;
    strcpy(mainwindow->signalcomp[row]->signallabel, mainwindow->signalcomp[row]->signallabel_bu);
    mainwindow->signalcomp[row]->signallabellen = mainwindow->signalcomp[row]->signallabellen_bu;
    strcpy(mainwindow->signalcomp[row]->physdimension, mainwindow->signalcomp[row]->physdimension_bu);
  }

  for(j=0; j<mainwindow->signalcomp[row]->fidfilter_cnt; j++)
  {
    free(mainwindow->signalcomp[row]->fidfilter[j]);
    fid_run_free(mainwindow->signalcomp[row]->fid_run[j]);
    fid_run_freebuf(mainwindow->signalcomp[row]->fidbuf[j]);
    fid_run_freebuf(mainwindow->signalcomp[row]->fidbuf2[j]);
  }

  mainwindow->signalcomp[row]->fidfilter_cnt = 0;

  free(mainwindow->signalcomp[row]);

  for(i=row; i<mainwindow->signalcomps - 1; i++)
  {
    mainwindow->signalcomp[i] = mainwindow->signalcomp[i + 1];
  }

  mainwindow->signalcomps--;

  mainwindow->setup_viewbuf();
}


void UI_SignalChooser::signalInvert()
{
  int i,
      row,
      size;

  char str[512];


  size = list->count();

  if(size < 1)
  {
    return;
  }

  row = list->currentRow();

  for(i=0; i<mainwindow->signalcomp[row]->num_of_signals; i++)
  {
    mainwindow->signalcomp[row]->factor[i] *= -1;

    strcpy(str, mainwindow->signalcomp[row]->edfhdr->edfparam[mainwindow->signalcomp[row]->edfsignal[i]].label);

    strip_types_from_label(str);

    if(i == 0)
    {
      if(mainwindow->signalcomp[row]->factor[i] < 0)
      {
        sprintf(mainwindow->signalcomp[row]->signallabel,
                "- %s",
                str);
      }
      else
      {
        sprintf(mainwindow->signalcomp[row]->signallabel,
                "%s",
                str);
      }
    }
    else
    {
      if(mainwindow->signalcomp[row]->factor[i] < 0)
      {
        sprintf(mainwindow->signalcomp[row]->signallabel + strlen(mainwindow->signalcomp[row]->signallabel),
                " - %s",
                str);
      }
      else
      {
        sprintf(mainwindow->signalcomp[row]->signallabel + strlen(mainwindow->signalcomp[row]->signallabel),
                " + %s",
                str);
      }
    }

    remove_trailing_spaces(mainwindow->signalcomp[row]->signallabel);

    mainwindow->signalcomp[row]->signallabellen = strlen(mainwindow->signalcomp[row]->signallabel);

    if(mainwindow->signalcomp[row]->signallabellen > 480) break;
  }

  if(mainwindow->signalcomp[row]->ecg_filter != NULL)
  {
    reset_ecg_filter(0.0, mainwindow->signalcomp[row]->ecg_filter);

    strcpy(mainwindow->signalcomp[row]->signallabel_bu, mainwindow->signalcomp[row]->signallabel);

    strcpy(mainwindow->signalcomp[row]->signallabel, "HR");

    mainwindow->signalcomp[row]->signallabellen_bu = mainwindow->signalcomp[row]->signallabellen;

    mainwindow->signalcomp[row]->signallabellen = strlen(mainwindow->signalcomp[row]->signallabel);
  }

  list->item(row)->setText(mainwindow->signalcomp[row]->signallabel);

  mainwindow->setup_viewbuf();
}


void UI_SignalChooser::strip_types_from_label(char *label)
{
  int i,
      len;


  len = strlen(label);
  if(len<16)
  {
    return;
  }

  if((!(strncmp(label, "EEG ", 4)))
   ||(!(strncmp(label, "ECG ", 4)))
   ||(!(strncmp(label, "EOG ", 4)))
   ||(!(strncmp(label, "ERG ", 4)))
   ||(!(strncmp(label, "EMG ", 4)))
   ||(!(strncmp(label, "MEG ", 4)))
   ||(!(strncmp(label, "MCG ", 4))))
  {
    if(label[4]!=' ')
    {
      for(i=0; i<(len-4); i++)
      {
        label[i] = label[i+4];
      }

      for(; i<len; i++)
      {
        label[i] = ' ';
      }
    }
  }
}






